//$Id: popup-trigger-canvas.cc,v 1.24 2004/01/20 22:52:15 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001-2003 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "popup-trigger-canvas.h"

#include "form-editor.h"
#include "form-editor-canvas.h"
#include "resizeable-ops.h"

#include <gnome--/canvas-polygon.h>
#include <gnome--/canvas-rect.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk-pixbuf/gnome-canvas-pixbuf.h>

#include "pixmaps/popup-trigger-canvas.xpm"
#include "pixmaps/popup-trigger-canvas-disabled.xpm"

using namespace Guikachu::GUI::FormEditor;

PopupTriggerCanvasItem::PopupTriggerCanvasItem (Widgets::PopupTrigger *widget_,
						Gnome::CanvasGroup    *parent_group):
    CanvasItem (widget_, parent_group),
    WidgetCanvasItem (widget_, parent_group),
    ResizeableCanvasItem (widget_, parent_group, new WidgetOps::ResizeOpFactory (widget_)),
    widget (widget_),
    group (0)
{
    widget->bitmap_id.resource_changed.connect (
	SigC::slot (this, &PopupTriggerCanvasItem::update));
    update ();
}

void PopupTriggerCanvasItem::update ()
{
    if (group)
	delete group;

    group = new Gnome::CanvasGroup (*draw_group, 0, 0);

    if (widget->bitmap_id == "" && widget->selected_bitmap_id == "")
        draw_textual ();
    else
        draw_bitmap ();

    set_canvas_item (group);
}

void PopupTriggerCanvasItem::draw_textual ()
{
    GdkPixbuf *pixbuf = 0;
    if (widget->usable)
	pixbuf = gdk_pixbuf_new_from_xpm_data ((const char**)popup_trigger_canvas_xpm);
    else
	pixbuf = gdk_pixbuf_new_from_xpm_data ((const char**)popup_trigger_canvas_disabled_xpm);

    gnome_canvas_item_new (group->gtkobj (),
			   gnome_canvas_pixbuf_get_type (),
			   "pixbuf", pixbuf,
			   "x", (double)widget->x,
			   "y", (double)widget->y,
			   0);
    gdk_pixbuf_unref (pixbuf);
    
    int width = widget->get_width ();
    int height = widget->get_height ();

    int text_x = widget->x + 15;
    int text_y = widget->y;

    draw_text (*group,
               widget->text, get_font (widget->font),
               get_foreground_color (widget->usable),
               text_x, text_y,
               width - 15, height,
               GTK_ANCHOR_NW);
}

void PopupTriggerCanvasItem::draw_bitmap ()
{
    FormEditor::draw_bitmapref (*group, widget->bitmap_id,
                                widget->get_manager ()->get_target ()->screen_color,
                                get_background_color (),
                                widget->x + 3, widget->y,
                                widget->get_width () - 3, widget->get_height (),
                                GTK_ANCHOR_CENTER);
}

void PopupTriggerCanvasItem::get_bounds (int &x1, int &y1,
					 int &x2, int &y2)
{
    x1 = widget->x;
    x2 = x1 + widget->get_width ();
    y1 = widget->y;
    y2 = y1 + widget->get_height ();
}
