//$Id: top-treeitem.cc,v 1.9 2003/06/30 14:07:56 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001-2003 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "top-treeitem.h"

#include "config.h"
#include <libgnome/libgnome.h>

#include <gnome--/app-helper.h>
#include <libgnomeui/gnome-stock.h>
#include <list>

#include "resource-manager.h"

using namespace Guikachu;

GUI::TopTreeItem::TopTreeItem (Resources::Application *res_):
    res (res_),
    res_gui (0)
{
    button_press_event.connect (slot (this, &GUI::TopTreeItem::clicked_cb));
    key_press_event.connect (slot (this, &GUI::TopTreeItem::key_press_cb));

    res->changed.connect (slot (this, &GUI::TopTreeItem::application_changed_cb));

    using Gnome::UI;
    
    std::list <UI::Info> popup_menu_list;
    popup_menu_list.push_back (
	UI::Item (UI::Icon (GNOME_STOCK_MENU_PROP),
		  _("_Edit"), SigC::slot (this, &GUI::TopTreeItem::menu_edit_cb)));
    UI::fill (popup_menu, popup_menu_list, *(popup_menu.get_accel_group ()));

    application_changed_cb ();
}

GUI::TopTreeItem::~TopTreeItem ()
{
    delete res_gui;
}

int GUI::TopTreeItem::clicked_cb (GdkEventButton *e)
{
    if (e->type == GDK_2BUTTON_PRESS && e->button == 1)
	show_ui ();

    if (e->button == 3)
	popup_menu.popup (e->button, e->time);

    return false;
}

int GUI::TopTreeItem::key_press_cb (GdkEventKey *e)
{
    switch (e->keyval)
    {
    case GDK_Return:
    case GDK_KP_Enter:
	show_ui ();
	break;
	
    case GDK_Delete:
    case GDK_KP_Delete:
	res->reset ();
	if (res_gui)
	    res_gui->hide ();
	break;
	
    default:
	break;
    }

    return true;
}

void GUI::TopTreeItem::application_changed_cb ()
{
    remove ();

    if (res->iconname != "")
	add_label (res->iconname);
    else
	add_label (_("Application"));
}

void GUI::TopTreeItem::menu_edit_cb ()
{
    show_ui ();
}

void GUI::TopTreeItem::show_ui ()
{
    if (!res_gui)
	res_gui = new AppWindow (res);

    res_gui->show ();
}
