//$Id: treeitem.cc,v 1.18 2003/06/30 14:07:56 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001-2003 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "treeitem.h"

#include "config.h"
#include <libgnome/libgnome.h>

#include "resource-manager-ops.h"
#include "edit-cut-and-paste.h"

#include <gnome--/app-helper.h>
#include <libgnomeui/gnome-stock.h>
#include <list>

#include "resource-factory.h"

using namespace Guikachu;
using namespace Gnome;

GUI::TreeItem::TreeItem (Resource *res_):
    Gtk::TreeItem (res_->id),
    res (res_),
    res_win (0)
{
    button_press_event.connect (slot (this, &GUI::TreeItem::button_press_cb));
    key_press_event.connect (slot (this, &GUI::TreeItem::key_press_cb));
    
    res->changed.connect (slot (this, &GUI::TreeItem::resource_changed_cb));
    
    std::list <UI::Info> popup_menu_list;
    popup_menu_list.push_back (UI::Item (
	UI::Icon (GNOME_STOCK_MENU_PROP), _("_Edit"),
	SigC::slot (this, &GUI::TreeItem::menu_edit_cb)));
    popup_menu_list.push_back (UI::Separator ());
    popup_menu_list.push_back (MenuItems::Cut (SigC::slot (this, &GUI::TreeItem::menu_cut_cb)));
    popup_menu_list.push_back (MenuItems::Copy (SigC::slot (this, &GUI::TreeItem::menu_copy_cb)));
    popup_menu_list.push_back (UI::Separator ());
    popup_menu_list.push_back (UI::Item (
	UI::Icon (GNOME_STOCK_MENU_TRASH), _("_Remove"),
	SigC::slot (this, &GUI::TreeItem::menu_remove_cb)));

    UI::fill (popup_menu, popup_menu_list, *(popup_menu.get_accel_group ()));
}

int GUI::TreeItem::key_press_cb (GdkEventKey *e)
{
    switch (e->keyval)
    {
    case GDK_Return:
    case GDK_KP_Enter:
	show_ui ();
	break;
	
    case GDK_Delete:
    case GDK_KP_Delete:	
	menu_remove_cb ();
	break;
	
    default:
	break;
    }

    return true;
}

int GUI::TreeItem::button_press_cb (GdkEventButton *e)
{
    if (e->type == GDK_2BUTTON_PRESS && e->button == 1)
	show_ui ();

    if (e->button == 3)
	popup_menu.popup (e->button, e->time);
    
    return false;
    
}

void GUI::TreeItem::resource_changed_cb ()
{
    remove ();
    add_label (res->id);
}

void GUI::TreeItem::menu_remove_cb ()
{
    ResourceManager *manager = res->get_manager ();
    
    manager->get_undo_manager ().push (new ResourceOps::RemoveOp (res));
    manager->remove_resource (res);
}

void GUI::TreeItem::menu_edit_cb ()
{
    show_ui ();
}

void GUI::TreeItem::menu_cut_cb ()
{
    menu_copy_cb ();
    menu_remove_cb ();
}

void GUI::TreeItem::menu_copy_cb ()
{
    Edit::copy_resource (res);
}

void GUI::TreeItem::show_ui ()
{
    if (!res_win)
	res_win = get_resource_editor (res);

    res_win->show ();
}
